@@ -2,10 +2,11 @@
 #this repository contains the full copyright notices and license terms.
 import datetime
 from threading import Lock
+from collections import OrderedDict
+
 from trytond.transaction import Transaction
 from trytond.config import CONFIG
-from trytond.backend import Database
-from trytond.tools import OrderedDict
+from trytond import backend
 
 __all__ = ['Cache', 'LRUDict']
 
@@ -63,6 +64,7 @@ class Cache(object):
                 cache[key] = value
             except TypeError:
                 pass
+        return value
 
     def clear(self):
         cursor = Transaction().cursor
@@ -74,7 +76,7 @@ class Cache(object):
     def clean(dbname):
         if not CONFIG['multi_server']:
             return
-        database = Database(dbname).connect()
+        database = backend.get('Database')(dbname).connect()
         cursor = database.cursor()
         try:
             cursor.execute('SELECT "timestamp", "name" FROM ir_cache')
@@ -104,7 +106,7 @@ class Cache(object):
     def resets(dbname):
         if not CONFIG['multi_server']:
             return
-        database = Database(dbname).connect()
+        database = backend.get('Database')(dbname).connect()
         cursor = database.cursor()
         try:
             with Cache._resets_lock:
