@@ -1,10 +1,11 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 import os
+import sys
 import logging
 from lxml import etree
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.backend import TableHandler
+from trytond import backend
 from trytond.pyson import CONTEXT, Eval, Bool, PYSONDecoder
 from trytond.tools import safe_eval, file_open
 from trytond.transaction import Transaction
@@ -15,7 +16,7 @@ from trytond.rpc import RPC
 
 __all__ = [
     'View', 'ShowViewStart', 'ShowView',
-    'ViewTreeWidth', 'ViewTreeExpandedState', 'ViewSearch',
+    'ViewTreeWidth', 'ViewTreeState', 'ViewSearch',
     ]
 
 
@@ -32,6 +33,7 @@ class View(ModelSQL, ModelView):
             ('tree', 'Tree'),
             ('form', 'Form'),
             ('graph', 'Graph'),
+            ('calendar', 'Calendar'),
             ('board', 'Board'),
             ], 'View Type', select=True)
     data = fields.Text('Data')
@@ -62,11 +64,14 @@ class View(ModelSQL, ModelView):
                 })
         cls._order.insert(0, ('priority', 'ASC'))
         cls._buttons.update({
-                'show': {},
+                'show': {
+                    'readonly': Eval('type') != 'form',
+                    },
                 })
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
 
@@ -100,7 +105,9 @@ class View(ModelSQL, ModelView):
         key = (cls.__name__, type_)
         rng = cls._get_rng_cache.get(key)
         if rng is None:
-            rng_name = os.path.join(os.path.dirname(__file__), type_ + '.rng')
+            rng_name = os.path.join(os.path.dirname(
+                    unicode(__file__, sys.getfilesystemencoding())),
+                type_ + '.rng')
             rng = etree.fromstring(open(rng_name).read())
             cls._get_rng_cache.set(key, rng)
         return rng
@@ -136,15 +143,10 @@ class View(ModelSQL, ModelView):
             # validate pyson attributes
             validates = {
                 'states': fields.states_validate,
-                'domain': fields.domain_validate,
-                'context': fields.context_validate,
-                'digits': fields.digits_validate,
-                'add_remove': fields.add_remove_validate,
             }
 
             def encode(element):
-                for attr in ('states', 'domain', 'context', 'digits',
-                        'add_remove', 'spell', 'colors'):
+                for attr in ('states', 'domain', 'spell', 'colors'):
                     if element.get(attr):
                         try:
                             value = safe_eval(element.get(attr), CONTEXT)
@@ -163,8 +165,6 @@ class View(ModelSQL, ModelView):
             if not encode(root_element):
                 cls.raise_user_error('invalid_xml', (view.rec_name,))
 
-        return True
-
     def get_arch(self, name):
         value = None
         if self.name and self.module:
@@ -196,8 +196,8 @@ class View(ModelSQL, ModelView):
         return views
 
     @classmethod
-    def write(cls, views, vals):
-        super(View, cls).write(views, vals)
+    def write(cls, views, values, *args):
+        super(View, cls).write(views, values, *args)
         # Restart the cache
         ModelView._fields_view_get_cache.clear()
 
@@ -221,8 +221,6 @@ class ShowView(Wizard):
             View = pool.get('ir.ui.view')
             view = View(Transaction().context.get('active_id'))
             Model = pool.get(view.model)
-            if view.type != 'form':
-                return Model.fields_view_get(view_type='form')
             return Model.fields_view_get(view_id=view.id)
 
         def get_defaults(self, wizard, state_name, fields):
@@ -262,8 +260,8 @@ class ViewTreeWidth(ModelSQL, ModelView):
         return res
 
     @classmethod
-    def write(cls, records, vals):
-        super(ViewTreeWidth, cls).write(records, vals)
+    def write(cls, records, values, *args):
+        super(ViewTreeWidth, cls).write(records, values, *args)
         ModelView._fields_view_get_cache.clear()
 
     @classmethod
@@ -291,9 +289,9 @@ class ViewTreeWidth(ModelSQL, ModelView):
             cls.create(to_create)
 
 
-class ViewTreeExpandedState(ModelSQL, ModelView):
-    'View Tree Expanded State'
-    __name__ = 'ir.ui.view_tree_expanded_state'
+class ViewTreeState(ModelSQL, ModelView):
+    'View Tree State'
+    __name__ = 'ir.ui.view_tree_state'
     _rec_name = 'model'
     model = fields.Char('Model', required=True)
     domain = fields.Char('Domain', required=True)
@@ -301,20 +299,28 @@ class ViewTreeExpandedState(ModelSQL, ModelView):
             ondelete='CASCADE')
     child_name = fields.Char('Child Name')
     nodes = fields.Text('Expanded Nodes')
+    selected_nodes = fields.Text('Selected Nodes')
 
     @classmethod
     def __setup__(cls):
-        super(ViewTreeExpandedState, cls).__setup__()
+        super(ViewTreeState, cls).__setup__()
         cls.__rpc__.update({
-                'set_expanded': RPC(readonly=False),
-                'get_expanded': RPC(),
+                'set': RPC(readonly=False),
+                'get': RPC(),
                 })
 
     @classmethod
     def __register__(cls, module_name):
-        super(ViewTreeExpandedState, cls).__register__(module_name)
-
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
+        table = TableHandler(cursor, cls, module_name)
+
+        # Migration from 2.8: table name changed
+        table.table_rename(cursor, 'ir_ui_view_tree_expanded_state',
+            cls._table)
+
+        super(ViewTreeState, cls).__register__(module_name)
+
         table = TableHandler(cursor, cls, module_name)
         table.index_action(['model', 'domain', 'user', 'child_name'], 'add')
 
@@ -322,8 +328,12 @@ class ViewTreeExpandedState(ModelSQL, ModelView):
     def default_nodes():
         return '[]'
 
+    @staticmethod
+    def default_selected_nodes():
+        return '[]'
+
     @classmethod
-    def set_expanded(cls, model, domain, child_name, nodes):
+    def set(cls, model, domain, child_name, nodes, selected_nodes):
         current_user = Transaction().user
         with Transaction().set_user(0):
             records = cls.search([
@@ -339,10 +349,11 @@ class ViewTreeExpandedState(ModelSQL, ModelView):
                         'domain': domain,
                         'child_name': child_name,
                         'nodes': nodes,
+                        'selected_nodes': selected_nodes,
                         }])
 
     @classmethod
-    def get_expanded(cls, model, domain, child_name):
+    def get(cls, model, domain, child_name):
         current_user = Transaction().user
         with Transaction().set_user(0):
             try:
@@ -354,8 +365,10 @@ class ViewTreeExpandedState(ModelSQL, ModelView):
                         ],
                     limit=1)
             except ValueError:
-                return '[]'
-            return cls(expanded_info).nodes
+                return (cls.default_nodes(), cls.default_selected_nodes())
+            state = cls(expanded_info)
+            return (state.nodes or cls.default_nodes(),
+                state.selected_nodes or cls.default_selected_nodes())
 
 
 class ViewSearch(ModelSQL, ModelView):
