@@ -5,11 +5,13 @@ import traceback
 import logging
 import time
 import sys
-import hashlib
 import pydoc
+
+from sql import Table, Flavor
+
 from trytond.pool import Pool
 from trytond import security
-from trytond.backend import Database, DatabaseOperationalError
+from trytond import backend
 from trytond.config import CONFIG
 from trytond.version import VERSION
 from trytond.transaction import Transaction
@@ -19,9 +21,16 @@ from trytond.exceptions import UserError, UserWarning, NotLogged, \
 from trytond.rpc import RPC
 
 
+ir_configuration = Table('ir_configuration')
+ir_lang = Table('ir_lang')
+ir_module = Table('ir_module')
+res_user = Table('res_user')
+
+
 def dispatch(host, port, protocol, database_name, user, session, object_type,
         object_name, method, *args, **kwargs):
-
+    Database = backend.get('Database')
+    DatabaseOperationalError = backend.get('DatabaseOperationalError')
     if object_type == 'common':
         if method == 'login':
             try:
@@ -47,8 +56,6 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
             return True
         elif method == 'version':
             return VERSION
-        elif method == 'timezone_get':
-            return CONFIG['timezone']
         elif method == 'list_lang':
             return [
                 ('bg_BG', 'Български'),
@@ -63,6 +70,7 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
                 ('lt_LT', 'Lietuvių'),
                 ('nl_NL', 'Nederlands'),
                 ('ru_RU', 'Russian'),
+                ('sl_SI', 'Slovenščina'),
             ]
         elif method == 'db_exist':
             try:
@@ -75,16 +83,8 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
         elif method == 'list':
             if CONFIG['prevent_dblist']:
                 raise Exception('AccessDenied')
-            database = Database().connect()
-            try:
-                cursor = database.cursor()
-                try:
-                    res = database.list(cursor)
-                finally:
-                    cursor.close(close=True)
-            except Exception:
-                res = []
-            return res
+            with Transaction().start(None, 0, close=True) as transaction:
+                return transaction.database.list(transaction.cursor)
         elif method == 'create':
             return create(*args, **kwargs)
         elif method == 'restore':
@@ -151,18 +151,18 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
         with Transaction().start(database_name, user,
                 readonly=rpc.readonly) as transaction:
             try:
-                args, kwargs, transaction.context, transaction.timestamp = \
-                    rpc.convert(obj, *args, **kwargs)
+                c_args, c_kwargs, transaction.context, transaction.timestamp \
+                    = rpc.convert(obj, *args, **kwargs)
                 meth = getattr(obj, method)
                 if not hasattr(meth, 'im_self') or meth.im_self:
-                    result = rpc.result(meth(*args, **kwargs))
+                    result = rpc.result(meth(*c_args, **c_kwargs))
                 else:
                     assert rpc.instantiate == 0
-                    inst = args.pop(0)
+                    inst = c_args.pop(0)
                     if hasattr(inst, method):
-                        result = rpc.result(meth(inst, *args, **kwargs))
+                        result = rpc.result(meth(inst, *c_args, **c_kwargs))
                     else:
-                        result = [rpc.result(meth(i, *args, **kwargs))
+                        result = [rpc.result(meth(i, *c_args, **c_kwargs))
                             for i in inst]
                 if not rpc.readonly:
                     transaction.cursor.commit()
@@ -183,17 +183,16 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
                             database_name) + tb_s)
                 transaction.cursor.rollback()
                 raise
-        if not (object_name == 'res.request' and method == 'request_get'):
-            with Transaction().start(database_name, 0) as transaction:
-                pool = Pool(database_name)
-                Session = pool.get('ir.session')
-                try:
-                    Session.reset(session)
-                except DatabaseOperationalError:
-                    # Silently fail when reseting session
-                    transaction.cursor.rollback()
-                else:
-                    transaction.cursor.commit()
+        with Transaction().start(database_name, 0) as transaction:
+            pool = Pool(database_name)
+            Session = pool.get('ir.session')
+            try:
+                Session.reset(session)
+            except DatabaseOperationalError:
+                # Silently fail when reseting session
+                transaction.cursor.rollback()
+            else:
+                transaction.cursor.commit()
         Cache.resets(database_name)
         return result
 
@@ -208,49 +207,43 @@ def create(database_name, password, lang, admin_password):
     :param admin_password: the admin password
     :return: True if succeed
     '''
+    Database = backend.get('Database')
     security.check_super(password)
     res = False
     logger = logging.getLogger('database')
 
     try:
-        database = Database().connect()
-        cursor = database.cursor(autocommit=True)
-        try:
-            database.create(cursor, database_name)
-            cursor.commit()
-            cursor.close(close=True)
-        except Exception:
-            cursor.close()
-            raise
+        with Transaction().start(None, 0, close=True, autocommit=True) \
+            as transaction:
+            transaction.database.create(transaction.cursor, database_name)
+            transaction.cursor.commit()
 
         with Transaction().start(database_name, 0) as transaction:
-            database.init(transaction.cursor)
-            transaction.cursor.execute('INSERT INTO ir_configuration '
-                '(language) VALUES (%s)', (lang,))
+            Database.init(transaction.cursor)
+            transaction.cursor.execute(*ir_configuration.insert(
+                    [ir_configuration.language], [[lang]]))
             transaction.cursor.commit()
 
         pool = Pool(database_name)
         pool.init(update=True, lang=[lang])
         with Transaction().start(database_name, 0) as transaction:
-            cursor = transaction.cursor
-            #XXX replace with model write
-            cursor.execute('UPDATE ir_lang '
-                'SET translatable = %s '
-                'WHERE code = %s', (True, lang))
-            cursor.execute('UPDATE res_user '
-                'SET language = (' +
-                cursor.limit_clause('SELECT id FROM ir_lang '
-                    'WHERE code = %s', 1) + ')'
-                'WHERE login <> \'root\'', (lang,))
-            admin_password = hashlib.sha1(admin_password.encode('utf-8'))\
-                .hexdigest()
-            cursor.execute('UPDATE res_user '
-                'SET password = %s '
-                'WHERE login = \'admin\'', (admin_password,))
+            User = pool.get('res.user')
+            Lang = pool.get('ir.lang')
+            language, = Lang.search([('code', '=', lang)])
+            language.translatable = True
+            language.save()
+            users = User.search([('login', '!=', 'root')])
+            User.write(users, {
+                    'language': language.id,
+                    })
+            admin, = User.search([('login', '=', 'admin')])
+            User.write([admin], {
+                    'password': admin_password,
+                    })
             Module = pool.get('ir.module.module')
             if Module:
                 Module.update_list()
-            cursor.commit()
+            transaction.cursor.commit()
             res = True
     except Exception:
         logger.error('CREATE DB: %s failed' % (database_name,))
@@ -263,17 +256,18 @@ def create(database_name, password, lang, admin_password):
 
 
 def drop(database_name, password):
+    Database = backend.get('Database')
     security.check_super(password)
     Database(database_name).close()
     # Sleep to let connections close
     time.sleep(1)
     logger = logging.getLogger('database')
 
-    database = Database().connect()
-    cursor = database.cursor(autocommit=True)
-    try:
+    with Transaction().start(None, 0, close=True, autocommit=True) \
+        as transaction:
+        cursor = transaction.cursor
         try:
-            database.drop(cursor, database_name)
+            Database.drop(cursor, database_name)
             cursor.commit()
         except Exception:
             logger.error('DROP DB: %s failed' % (database_name,))
@@ -283,12 +277,11 @@ def drop(database_name, password):
         else:
             logger.info('DROP DB: %s' % (database_name))
             Pool.stop(database_name)
-    finally:
-        cursor.close(close=True)
     return True
 
 
 def dump(database_name, password):
+    Database = backend.get('Database')
     security.check_super(password)
     Database(database_name).close()
     # Sleep to let connections close
@@ -301,6 +294,7 @@ def dump(database_name, password):
 
 
 def restore(database_name, password, data, update=False):
+    Database = backend.get('Database')
     logger = logging.getLogger('database')
     security.check_super(password)
     try:
@@ -314,12 +308,11 @@ def restore(database_name, password, data, update=False):
     logger.info('RESTORE DB: %s' % (database_name))
     if update:
         cursor = Database(database_name).connect().cursor()
-        cursor.execute('SELECT code FROM ir_lang '
-            'WHERE translatable')
+        cursor.execute(*ir_lang.select(ir_lang.code,
+                where=ir_lang.translatable))
         lang = [x[0] for x in cursor.fetchall()]
-        cursor.execute('UPDATE ir_module_module SET '
-            "state = 'to upgrade' "
-            "WHERE state = 'installed'")
+        cursor.execute(*ir_module.update([ir_module.state], ['to upgrade'],
+                where=(ir_module.state == 'installed')))
         cursor.commit()
         cursor.close()
         Pool(database_name).init(update=update, lang=lang)
