@@ -185,10 +185,10 @@ class And(PYSON):
         for statement in statements:
             if isinstance(statement, PYSON):
                 assert statement.types() == set([bool]), \
-                        'statement must be boolean'
+                    'statement must be boolean'
             else:
                 assert isinstance(statement, bool), \
-                        'statement must be boolean'
+                    'statement must be boolean'
         assert len(statements) >= 2, 'must have at least 2 statements'
         self._statements = list(statements)
 
@@ -256,10 +256,10 @@ class Greater(PYSON):
         for i in (statement1, statement2):
             if isinstance(i, PYSON):
                 assert i.types().issubset(set([int, long, float])), \
-                        'statement must be an integer or a float'
+                    'statement must be an integer or a float'
             else:
                 assert isinstance(i, (int, long, float)), \
-                        'statement must be an integer or a float'
+                    'statement must be an integer or a float'
         if isinstance(equal, PYSON):
             assert equal.types() == set([bool])
         else:
@@ -327,10 +327,10 @@ class If(PYSON):
             then_types = set([type(then_statement)])
         if isinstance(else_statement, PYSON):
             assert then_types == else_statement.types(), \
-                    'then and else statements must be the same type'
+                'then and else statements must be the same type'
         else:
             assert then_types == set([type(else_statement)]), \
-                    'then and else statements must be the same type'
+                'then and else statements must be the same type'
         self._condition = condition
         self._then_statement = then_statement
         self._else_statement = else_statement
@@ -398,13 +398,13 @@ class In(PYSON):
         super(In, self).__init__()
         if isinstance(key, PYSON):
             assert key.types().issubset(set([str, int, long])), \
-                    'key must be a string or an integer or a long'
+                'key must be a string or an integer or a long'
         else:
             assert type(key) in [str, int, long], \
-                    'key must be a string or an integer or a long'
+                'key must be a string or an integer or a long'
         if isinstance(obj, PYSON):
             assert obj.types().issubset(set([dict, list])), \
-                    'obj must be a dict or a list'
+                'obj must be a dict or a list'
             if obj.types() == set([dict]):
                 assert type(key) == str, 'key must be a string'
         else:
@@ -437,10 +437,10 @@ class Date(PYSON):
         for i in (year, month, day, delta_years, delta_months, delta_days):
             if isinstance(i, PYSON):
                 assert i.types().issubset(set([int, long, type(None)])), \
-                        '%s must be an integer or None' % (i,)
+                    '%s must be an integer or None' % (i,)
             else:
                 assert isinstance(i, (int, long, type(None))), \
-                        '%s must be an integer or None' % (i,)
+                    '%s must be an integer or None' % (i,)
         self._year = year
         self._month = month
         self._day = day
@@ -488,10 +488,10 @@ class DateTime(Date):
                 delta_hours, delta_minutes, delta_seconds, delta_microseconds):
             if isinstance(i, PYSON):
                 assert i.types() == set([int, long, type(None)]), \
-                        '%s must be an integer or None' % (i,)
+                    '%s must be an integer or None' % (i,)
             else:
                 assert isinstance(i, (int, long, type(None))), \
-                        '%s must be an integer or None' % (i,)
+                    '%s must be an integer or None' % (i,)
         self._hour = hour
         self._minute = minute
         self._second = second
@@ -537,6 +537,32 @@ class DateTime(Date):
             )
 
 
+class Len(PYSON):
+
+    def __init__(self, value):
+        super(Len, self).__init__()
+        if isinstance(value, PYSON):
+            assert value.types().issubset(set([dict, list, str])), \
+                'value must be a dict or a list or a string'
+        else:
+            assert type(value) in [dict, list, str], \
+                'value must be a dict or list or a string'
+        self._value = value
+
+    def pyson(self):
+        return {
+            '__class__': 'Len',
+            'v': self._value,
+            }
+
+    def types(self):
+        return set([int, long])
+
+    @staticmethod
+    def eval(dct, context):
+        return len(dct['v'])
+
+
 class Id(PYSON):
     """The database id for filesystem id"""
 
@@ -567,4 +593,5 @@ CONTEXT = {
     'In': In,
     'Date': Date,
     'DateTime': DateTime,
+    'Len': Len,
 }
